/*
 *  Author:  Daniel De Graaf <dgdegra@tycho.nsa.gov>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2,
 *  as published by the Free Software Foundation.
 */

#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <xenctrl.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <string.h>
#include <unistd.h>
#include <inttypes.h>
#define XSM_BOOLEAN_CONFIG_PATH "/config/xsm/boolean/"

static void usage(char **argv)
{
	fprintf(stderr, "Usage: %s name value\n", argv[0]);
	exit(1);
}

static int str2bool(const char *str)
{
	if (str[0] == '0' || str[0] == '1')
		return (str[0] == '1');
	if (!strcasecmp(str, "enabled") || !strcasecmp(str, "on") || !strcasecmp(str, "y"))
		return 1;
	if (!strcasecmp(str, "disabled") || !strcasecmp(str, "off") || !strcasecmp(str, "n"))
		return 0;
	fprintf(stderr, "Unknown value %s\n", str);
	exit(1);
}

static int persistbool(char *name, int value)
{
	int fd, len, ret;
	char buf[2], *fname;
	
	if (value < 0 || value > 1) {
		return -1;
	}

	len = strlen(name) + strlen(XSM_BOOLEAN_CONFIG_PATH) + 1;
	fname = (char *)malloc(sizeof(char) *len);
	if (!fname)
		return -1;
	snprintf(fname, len, "%s%s", XSM_BOOLEAN_CONFIG_PATH, name);
	fd = open(fname, O_WRONLY | O_CREAT | O_TRUNC, 0666);
	if (fd < 0) {
		fprintf(stderr, "\nxc_flask_setbool: Failed to write boolean file : %s, %s\n", fname,strerror(errno));
		free(fname);
		return -1;
	}
	// write the new bool value to file.
	snprintf(buf, sizeof(buf), "%d", value);
	
	ret = write(fd, buf, sizeof(buf));
	close(fd);
	free(fname);
	return ret;
}

int main(int argc, char **argv)
{
	int err = 0;
	xc_interface *xch;
	int value;

	if (argc != 3)
		usage(argv);

	value = str2bool(argv[2]);

	xch = xc_interface_open(0,0,0);
	if ( !xch )
	{
		fprintf(stderr, "Unable to create interface to xenctrl: %s\n",
				strerror(errno));
		err = 1;
		goto done;
	}

	err = xc_flask_setbool(xch, argv[1], value, 1);
	if (err) {
		fprintf(stderr, "xc_flask_setbool: Unable to set boolean %s=%s: %s (%d)",
			argv[1], argv[2], strerror(errno), err);
		err = 2;
		goto done;
	}
	err = persistbool(argv[1], value);
	if(err < 0){ 
		 fprintf(stderr, "xc_flask_setbool: Unable to persist boolean %s=%s: %s (%d)",
                        argv[1], argv[2], strerror(errno), err);
	}

 done:
	if ( xch )
		xc_interface_close(xch);

	return err;
}
